﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace CreaStelline
{
    // Classe contenente informazioni sull'animazione
    // come FramePerSecond, velocità angolare, 
    // l'immagine contenente i frame e altro ancora.
    class Animation: ICloneable {

        // Numero di volte che l'animazione è stata ripetuta
        private int repeated;
        public int Repeated { get { return repeated; } }
        
        // Tempo in cui l'animazione è iniziata
        private TimeSpan startTime;
        
        // Numero massimo di ripetizioni dell'animazioni
        // dopo le quali Update restituirà false
        private int maxRepeat = int.MaxValue;
        
        // Angolo di rotazione corrente in radianti
        private double currentRotation = 0;
        public double CurrentRotation { get { return currentRotation; } }
        
        // Indice del frame corrente
        private int currentFrame = -1;
        public int CurrentFrame { get { return currentFrame; } }
        
        
        

        // Proprietà per la velocità con cui si devono
        // succedere i frame
        public float fps { get; set; }
        // Immagine contenente tutti i frame
        public Texture2D Frames { get; set; }
        // Larghezza del singolo frame
        public int FrameWidth { get; set; }
        // Se true indica che l'animazione deve essere
        // riprodotta in senso inverso
        public bool Backward { get; set; }
        // Velocità angolare con cui deve ruotare l'animazione
        public double AngularVelocity { get; set; }
        // Angolo iniziale di rotazione
        public double InitalAngle { get; set; }
        // Punto rispetto al quale effettuare la rotazione
        public Vector2 RotationPoint { get; set; }
        // Il fattore di ingrandimento dell'animazione
        public float Scale { get; set; }


        public Animation(Texture2D Frames, int FrameWidth, float fps) : this(Frames, FrameWidth, fps, int.MaxValue) { }

        public Animation(Texture2D Frames, int FrameWidth, float fps, int MaxRepeat)
        {
            this.Frames = Frames;
            this.FrameWidth = FrameWidth;
            this.fps = fps;
            this.maxRepeat = MaxRepeat;
            this.InitalAngle = 0;
            this.Scale = 1.0f;

            // Di default in punto di rotazione è il centro 
            // dell'immagine
            this.RotationPoint = new Vector2(this.FrameWidth / 2, this.Frames.Height / 2);
        }


        // Proprietà che restituisce il rettangolo relativo
        // all'immagine contenente i frame da dissgnare
        public Rectangle CurrentRectangle {
            get {
                return new Rectangle(this.FrameWidth * this.currentFrame, 0, this.FrameWidth, this.Frames.Height);
            }
        }

        // Proprietà per conoscere il numero totale di frame
        public int TotalFrames { get { return (this.Frames.Width / this.FrameWidth); } }

        // Avvia l'animazione, vengono azzerati angolo di 
        // rotazione, numero di ripetizioni, frame corrente
        // e tempo di avvio
        public void Start(GameTime now) {
            this.repeated = 0;
            this.startTime = now.TotalGameTime;
            this.currentFrame = 0;
            this.currentRotation = 0;
        }

        // Aggiorna le variabili riguardanti l'animazione
        // a seconda del tempo trascorso
        public bool Update(GameTime now)
        {
            // Se non è ancora stato chiamato il metodo Start
            // lo chiamiamo ora
            if (this.currentFrame == -1) this.Start(now);
            // Calcoliamo il numero di secondi trascorsi 
                // dall'avvio dell'animazione
            double totalSeconds = (now.TotalGameTime - startTime).TotalSeconds;

            // Calcoliamo il numero totale di frame (teoricamente)
            // disegnati finora
            double frameDrawn = (totalSeconds * fps);

            // Calcoliamo il numero di volte che l'animazione è
            // stata riprodotta
            repeated = (int)Math.Truncate(frameDrawn / this.TotalFrames);

            // Calcoliamo il frame da disegnare prendendo il resto
            // della divisione del totale dei frame disegnati
            // per il numero di frame
            currentFrame = (int)Math.Truncate(frameDrawn % this.TotalFrames);

            // Calcoliamo l'angolo di rotazione corrente sommando
            // all'angolo iniziale il prodotto del tempo trascorso
            // per la velocità angolare
            currentRotation = InitalAngle + totalSeconds * AngularVelocity;

            // Per far scorrere l'animazione all'incontrario 
            // semplicemente prendiamo il corrisponde opposto
            if (this.Backward) currentFrame = this.TotalFrames - currentFrame - 1;

            // Restituisce true se non è stato superato il numero 
            // di ripetizioni massimo
            return (Repeated < maxRepeat);
        }


        // Implentiamo la clonazione dell'animazione
        #region ICloneable Members

        object ICloneable.Clone()
        {
            Animation clone = new Animation(this.Frames, this.FrameWidth, this.fps, this.maxRepeat);
            clone.AngularVelocity = this.AngularVelocity;
            clone.Backward = this.Backward;
            clone.InitalAngle = this.InitalAngle;
            clone.RotationPoint = this.RotationPoint;
            clone.Scale = this.Scale;
            return clone;
        }

        #endregion
    }
}
